import "./localStorage";

import { init, initializeLogin, finalizeLogin, credentialsProvider } from "@tidal-music/auth";

import { Webview } from "webview-bun";

const clientId = "mhPVJJEBNRzVjr2p";

export default async () => {
    await init({
        clientId,
        credentialsStorageKey: "tidal-credentials",
        scopes: ["r_usr", "w_usr"],
    })

    const credentials = await credentialsProvider.getCredentials();

    if (typeof credentials.userId !== "string" || (credentials.expires || 0) < Date.now()) {

        const response = await initializeLogin({
            redirectUri: "https://desktop.tidal.com/login/auth"
        });

        const webview = new Webview(true);

        webview.navigate(response);

        let didLogin = false;
        webview.bind("getURL", async (url: string) => {
            if (url.startsWith("https://desktop.tidal.com")) {
                const code = new URL(url)

                await finalizeLogin(code.search);
                didLogin = true;
            }
        });

        webview.init("getURL(location.href)");

        webview.run()

        while (!didLogin) {
            await new Promise((resolve) => setTimeout(resolve, 0));
        }

        webview.destroy()
    }

    return {
        "Authorization": `Bearer ${(await credentialsProvider.getCredentials()).token || ""}`
    }
}